﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;

namespace People.Types.DTO
{
    /// <summary>
    /// This is a view model of the Person business class. It contains fields and attributes specific to to GUI.
    /// The fields are named the same as the Person business class so we can take advantage of reflection to copy
    /// values back and forth between the view model and the business class, without directly referencing the business class.
    /// </summary>
    public class PersonViewModel
    {
        [Display(AutoGenerateField = false)]
        [Key()]
        public System.Nullable<System.Guid> PersonId { get; set; }

        [Display(AutoGenerateField = false)]
        public System.Nullable<System.Guid> AddressId { get; set; }

        [Display(Name = "First Name", Description = "Person's first name", Order = 1)]
        [Required()]
        public string FirstName { get; set; }

        [Display(Name = "Last Name", Description = "Person's last name", Order = 2)]
        [Required()]
        public string LastName { get; set; }

        [Display(Name = "Age", Description = "Person's age", Order = 3)]
        [RegularExpression("^[0-9]+", ErrorMessage = "Please enter age as a number.")]
        [Required()]
        public System.Nullable<int> Age { get; set; }

        [Display(Name = "Address", Order = 4)]
        public string AddressLine { get { if (Address.Street != null) { return Address.Street + " " + Address.City + ", " + Address.State; } else { return ""; } } }

        [Display(Name = "Date Added", Order = 5)]
        public DateTime CreateDate { get; set; }

        public AddressViewModel Address;

        public PersonViewModel()
        {
            Address = new AddressViewModel();
        }

        public PersonViewModel(object person) : this()
        {
            // Set our values.
            Converter.ConvertToBusinessObject<PersonViewModel>(this, person);
        }
    }
}
