using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using DB = People.DB;

/*
Primary Objects Linquify - Business Class Generator
Copyright (C) 2010 Kory Becker
http://www.primaryobjects.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

namespace People.Types.GeneratedTypes.BaseTypes
{
    [Serializable]
    public class Address : BusinessBaseType<DB.Address, DB.PeopleContextDataContext>
    {
        public class Where
        {
			public bool AddressIdSet;
			public bool CitySet;
			public bool StateSet;
			public bool StreetSet;

			private System.Nullable<System.Guid> _AddressId;
			private string _City;
			private string _State;
			private string _Street;


			public System.Nullable<System.Guid> AddressId { get { return _AddressId; } set { _AddressId = value; AddressIdSet = true; } }
			public string City { get { return _City; } set { _City = value; CitySet = true; } }
			public string State { get { return _State; } set { _State = value; StateSet = true; } }
			public string Street { get { return _Street; } set { _Street = value; StreetSet = true; } }


            public string MyWhere = "", MyOrderBy = "", MyTop = "";

            public Where()
            {
				AddressId = null;
				City = null;
				State = null;
				Street = null;

				AddressIdSet = false;
				CitySet = false;
				StateSet = false;
				StreetSet = false;

            }
        };

		public System.Guid AddressId { get; set; }
		public string City { get; set; }
		public string State { get; set; }
		public string Street { get; set; }

        public Where WhereClause { get; set; }

        #region Constructors

        public Address()
        {
            WhereClause = new Where();
        }

        public Address(System.Nullable<System.Guid> _AddressId, string _City, string _State, string _Street)
            : this()
        {
            if (_AddressId != null)
            {
                AddressId = (System.Guid)_AddressId;
            }

			City = _City;
			State = _State;
			Street = _Street;

        }

        public Address(DB.Address addressLinq)
            : this()
        {
            Address address = new Address();
            Converter.ConvertToBusinessObject<Address>(address, addressLinq);

            Initialize(address);
        }

        #endregion

        private void Initialize(Address address)
        {
			AddressId = address.AddressId;
			City = address.City;
			State = address.State;
			Street = address.Street;

            OnInitializeComplete();
        }

        #region Linq Helpers

        public bool Save()
        {
			OnSave();
			
            if (Save<System.Guid, Address>(this, AddressId, false))
            {
				OnSaveComplete();			
				return true;
			}
			
			return false;
        }

		public bool Insert()
		{
			// Allows you to force an Insert via the Save function, rather than an Update. Aka, explicit insert.
			// Useful for cases where you want to assign a primary key before inserting, rather than auto-assigning.
			OnSave();
			
            if (Save<System.Guid, Address>(this, AddressId, true))
            {
				OnSaveComplete();			
				return true;
			}
			
			return false;
		}
		
        public bool Load(System.Guid _AddressId)
        {
            DB.Address addressLinq = Load<System.Guid>(_AddressId);
            
            if (addressLinq == null)
            {
				return false;
            }
            
            Address address = new Address();
            Converter.ConvertToBusinessObject<Address>(address, addressLinq);
            
            Initialize(address);
            
            return true;
        }

        public bool Delete()
        {
	    OnDelete();

            DB.Address addressLinq = Delete<System.Guid>(AddressId);

			if (addressLinq == null)
			{
				return false;
			}
			
            Address address = new Address();
            Converter.ConvertToBusinessObject<Address>(address, addressLinq);

            Initialize(address);
            
	    OnDeleteComplete();

            return true;
        }

        public List<Address> Find()
        {
            List<DB.Address> listLinq = Find(WhereClause);
            
            List<Address> list = new List<Address>();
            foreach (DB.Address address in listLinq)
            {
                list.Add(new Address(address));
            }

            return list;
        }

        public new List<Address> ToList()
        {
            List<DB.Address> listLinq = base.ToList();

            List<Address> list = new List<Address>();
            foreach (DB.Address address in listLinq)
            {
                list.Add(new Address(address));
            }

            return list;
        }

        #endregion

        #region Overridables

        /// <summary>
        /// Fired after the object is created or loaded.
        /// </summary>
        protected virtual void OnInitializeComplete()
        {
			WhereClause.AddressId = AddressId;
        }

        /// <summary>
        /// Fired before the object is inserted or updated.
        /// </summary>
        protected virtual void OnSave()
        {
        }

        /// <summary>
        /// Fired after the object is inserted or updated.
        /// </summary>
        protected virtual void OnSaveComplete()
        {
			WhereClause.AddressId = AddressId;
        }

        /// <summary>
        /// Fired before the object is deleted.
        /// </summary>
	protected virtual void OnDelete()
	{
	}

        /// <summary>
        /// Fired after the object is deleted.
        /// </summary>
	protected virtual void OnDeleteComplete()
	{
	}

        #endregion
    }
}
