﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ServiceModel;
using People.Types.DTO;
using SilverlightApplication1.PeopleServiceClient;

namespace SilverlightApplication1
{
    public partial class MainPage : UserControl
    {
        private PeopleServiceClient.PeopleServiceClient _client = new PeopleServiceClient.PeopleServiceClient();

        public MainPage()
        {
            InitializeComponent();
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // Load the people from the web service.
            _client.GetPeopleCompleted += new EventHandler<GetPeopleCompletedEventArgs>(client_GetPeopleCompleted);
            _client.GetPeopleAsync();
        }

        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            // Dim the grid and show the DataForm.
            List<PersonViewModel> personList = new List<PersonViewModel>();
            personList.Add(new PersonViewModel());

            grdPeople.Opacity = 0.5;

            frmAddPerson.ItemsSource = personList;
            frmAddPerson.Visibility = Visibility.Visible;
            frmAddPerson.Focus();
        }

        private void frmAddPerson_EditEnded(object sender, DataFormEditEndedEventArgs e)
        {
            // The user closed the DataForm, save the person (new or update) via the web service and show the grid.
            if (e.EditAction == DataFormEditAction.Commit)
            {
                PersonViewModel personViewModel = (PersonViewModel)frmAddPerson.CurrentItem;

                _client.SavePersonCompleted += new EventHandler<SavePersonCompletedEventArgs>(client_SavePersonCompleted);
                _client.SavePersonAsync(personViewModel);
            }

            frmAddPerson.Visibility = Visibility.Collapsed;
            grdPeople.Opacity = 1;
        }

        private void btnDelete_Click(object sender, RoutedEventArgs e)
        {
            if (grdPeople.SelectedIndex > -1)
            {
                PersonViewModel personViewModel = (PersonViewModel)grdPeople.SelectedItem;

                _client.DeletePersonCompleted += new EventHandler<DeletePersonCompletedEventArgs>(client_DeletePersonCompleted);
                _client.DeletePersonAsync(personViewModel);
            }
            else
            {
                MessageBox.Show("Please select a person to delete.");
            }
        }

        private void btnEdit_Click(object sender, RoutedEventArgs e)
        {
            if (grdPeople.SelectedIndex > -1)
            {
                PersonViewModel personViewModel = (PersonViewModel)grdPeople.SelectedItem;

                // Load the selected person into the DataForm.
                List<PersonViewModel> personList = new List<PersonViewModel>();
                personList.Add(personViewModel);

                grdPeople.Opacity = 0.5;

                frmAddPerson.ItemsSource = personList;
                frmAddPerson.Visibility = Visibility.Visible;
                frmAddPerson.Focus();
            }
            else
            {
                MessageBox.Show("Please select a person to edit.");
            }
        }

        #region Asynchronous Action Event Handlers

        void client_GetPeopleCompleted(object sender, GetPeopleCompletedEventArgs e)
        {
            // Bind the grid.
            grdPeople.ItemsSource = e.Result;
        }

        void client_SavePersonCompleted(object sender, SavePersonCompletedEventArgs e)
        {
            // Bind the grid.
            UserControl_Loaded(sender, null);
        }

        void client_DeletePersonCompleted(object sender, DeletePersonCompletedEventArgs e)
        {
            // Bind the grid.
            UserControl_Loaded(sender, null);
        }

        #endregion
    }
}
